'' =================================================================================================
''
''   File....... stereo_dacs.spin
''   Purpose.... Audio DAC driver; injects noise for filtering by external RC
''   Author..... Chip Gracey, Parallax
''               -- modified by Jon McPhalen
''               -- see below for terms of use
''   E-mail..... 
''   Started.... 
''   Updated.... 19 MAY 2013
''
'' =================================================================================================


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28

  
var

  long  cog                                                     ' cog used by dacs driver

  long  samples                                                 ' hub address of samples
  long  lpin                                                    ' pins used for stereo output
  long  rpin
  long  dlevel


pub start(p_lrs, left, right, depth)

'' Starts dac driver on left and right pins
'' -- p_lrs is hub address of L/R samples pair (long w/two 16-bit samples)
'' -- left and right are pins used
''    * use -1 to disable a pin
'' -- depth is dither level depth (1 [most] to 8 [least])

  longmove(@samples, @p_lrs, 4)                                 ' copy parameters
  
  cog := cognew(@sdac, @samples) + 1                            

  return cog


pub startx(p_lrs, left, right, depth, cognum)

'' Start dac driver in specific cog
'' -- p_lrs is hub address of L/R samples pair (long w/ two 16-bit samples)
'' -- left and right are pins used
''    * use -1 to disable a pin
'' -- cognum is cog (0..7) to use
''    * select closest to IO pins used
'' -- depth is dither level depth (1 [most] to 8 [least]) 

  longmove(@samples, @p_lrs, 4)                                 ' copy parameters
  
  coginit(cognum, @sdac, @samples)                              ' start PASM driver 

  cog := cognum + 1
  

pub stop

'' Stops dacs cog if running

  if (cog)
    cogstop(cog - 1)
    cog := 0


dat                      

                        org     0

sdac                    mov     t1, par                         ' copy address of parameters
                        rdlong  smplspntr, t1                   ' read pointer to samples

getleft                 add     t1, #4                          ' point to left pin
                        rdlong  t2, t1                          ' read left pin
                        cmps    t2, #0                  wc      ' in use ( => 0)?
        if_b            jmp     #getright

setupleft               mov     t3, t2                          ' copy pin
                        add     t3, DUTY_SE                     ' setup for duty cycle mode
                        mov     ctra, t3                        ' enable counter
                        mov     frqa, #0
                        mov     t3, #1                          ' create mask for pin
                        shl     t3, t2
                        or      dira, t3                        ' set pin to output
                        
getright                add     t1, #4                          ' point to right pin
                        rdlong  t2, t1                          
                        cmps    t2, #0                  wc  
        if_b            jmp     #getdither

setupright              mov     t3, t2
                        add     t3, DUTY_SE
                        mov     ctrb, t3
                        mov     frqb, #0
                        mov     t3, #1
                        shl     t3, t2
                        or      dira, t3

getdither               add     t1, #4                          ' point to dither depth
                        rdlong  dadj, t1
                        min     dadj, #1
                        max     dadj, #8

dacloop                 rdlong  lsample, smplspntr              ' read stereo samples
                        xor     lsample, Hx80008000             ' convert to unsigned
                        
                        mov     rsample, lsample                ' copy, split and justify
                        shl     lsample, #16
                        and     rsample, HxFFFF0000             ' clean-up right

                        test    lfsr, taps              wc      ' iterate noise generator
                        rcl     lfsr, #1

                        mov     t1, lfsr                        ' make dither from noise
                        sar     t1, dadj                        ' adjust for depth setting

                        add     lsample, t1                     ' add dither to samples
                        add     rsample, t1

                        mov     frqa, lsample                   ' output samples
                        mov     frqb, rsample    

                        jmp     #dacloop

' --------------------------------------------------------------------------------------------------

DUTY_SE                 long    $1800_0000                      ' duty-cycle mode for counters

Hx80008000              long    $8000_8000
HxFFFF0000              long    $FFFF_0000

taps                    long    $8000_0062
lfsr                    long    1

smplspntr               res     1                               ' hub pointer to samples
lsample                 res     1                               ' left channel sample
rsample                 res     1                               ' right channel sample
dadj                    res     1                               ' dither adjust

t1                      res     1                               ' work vars
t2                      res     1
t3                      res     1
t4                      res     1

                        fit     496
                        

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}